==> flowsaber-0.1.3.2.tar.gz: flowsaber-0.1.3.2/README.md <==
# A dataflow based workflow framework.
### work in progress

<p align="center">

  

  
  <a href="https://pypi.python.org/pypi/flowsaber/">
    <img src="https://img.shields.io/pypi/v/flowsaber.svg" alt="Install with PyPi" />
  </a>
  
  
  <a href="https://github.com/zhqu1148980644/flowsaber/releases">
  	<img src="https://img.shields.io/github/v/release/zhqu1148980644/flowsaber?include_prereleases&label=github" alt="Github release">
  </a>
 
  <a href="https://zhqu1148980644.github.io/flowsaber/index.html">
  	<img src="https://readthedocs.org/projects/ansicolortags/badge/?version=latest" alt="Documentation">
  </a>
  
  <a href="https://pypi.python.org/pypi/flowsaber">
    <img src="https://img.shields.io/pypi/pyversions/flowsaber.svg" alt="Version">
  </a>
  
  <a href="https://pepy.tech/project/flowsaber">
    <img src="https://pepy.tech/badge/flowsaber" alt="Downloads">
  </a>

  <a href="https://pepy.tech/project/flowsaber">
    <img src="https://pepy.tech/badge/flowsaber/week" alt="Downloads per week">
  </a>
  
  <a href="https://github.com/zhqu1148980644/flowsaber/actions/workflows/python-package-conda.yml">
    <img src="https://github.com/zhqu1148980644/flowsaber/actions/workflows/python-package-conda.yml/badge.svg" alt="Build Status">
  </a>

  <a href="https://app.codecov.io/gh/zhqu1148980644/flowsaber">
    <img src="https://codecov.io/gh/zhqu1148980644/flowsaber/branch/main/graph/badge.svg" alt="codecov">
  </a>

  <a href="https://github.com/zhqu1148980644/flowsaber/blob/master/LICENSE">
    <img src="https://img.shields.io/github/license/zhqu1148980644/flowsaber" alt="license">
  </a>

</p>




### Features

- Dataflow-like task composing syntax inspired from [nextflow](https://github.com/nextflow-io/nextflow) 's DSL2.
- Python based: Import/Compose/Modify Task/Flow objects at any time.
- Local Cache.
- Clustering support based on ray.
- Conda and Container execution environment.


### Install

```bash
pip install flowsaber
```

### Example

- This is a rewrite version of [snakemake tutorial](https://snakemake.readthedocs.io/en/stable/tutorial/short.html)

```python
from flowsaber import *

# EnvTask is the real dependent task when using conda/image option
EnvTask.DEFAULT_CONFIG = {'workdir': '/tmp/Env'}  # make the EnvTask cache at a global place

@shell(conda="bwa=0.7.17 samtools=1.9")
def bwa(self, fa: File, fastq: File):  # input will be automatically converted if has type annotation
    """bwa mem -t {self.cpu} {fa} {fastq} | samtools view -Sb - > {fastq.stem}.bam"""
    return "*.bam"  # for ShellTask, str variable in the return will be treated as File and globed

@shell(conda="bwa=0.7.17 samtools=1.9")
def sort(bam: File):  # self is optional in case you don't want to access the current task
    """samtools sort -o {sorted_bam} {bam}"""  # use local variable
    sorted_bam = f"{bam.stem}.sorted.bam"
    return sorted_bam

@shell(conda="bcftools=1.9 samtools=1.9", pubdir="results/vcf")
def call(fa: File, bams: list):  # In case you need to write some python codes
    bams = ' '.join(str(bam) for bam in bams)
    Shell(f"samtools mpileup -g -f {fa} {bams} | bcftools call -mv - > all.vcf")
    return "all.vcf"

@task(pubdir="results/stats")
def stats(vcf: File):
    import matplotlib
    matplotlib.use("Agg")
    import matplotlib.pyplot as plt
    from pysam import VariantFile

    quals = [record.qual for record in VariantFile(vcf.open('rb'))]
    plt.hist(quals)

    plt.savefig("report.svg")

@flow
def call_vcf_flow(fa, fastq):
    def _call(bams):  # task is normal function, use python as wish
        return call(fa, bams)

    bam1 = bwa(fa, fastq)  # automatically clone channel
    bam2 = bwa(fa, fastq)
    mix(bam1, bam2) | sort | collect | _call | stats

config.update({
    'cpu': 8,
    'memory': 20,
})

prefix = 'snamke-demo.nosync/data'
fa = Channel.value(f'{prefix}/genome.fa')
fastq = Channel.values(*[f'{prefix}/samples/{sample}' for sample in ['A.fastq', 'B.fastq', 'C.fastq']])

runner, workflow = FlowRunner(call_vcf_flow).run(fa, fastq)
# generate dag
workflow.graph.render('/tmp/workflow', view=True, format='png', cleanup=True)
# run the flow
runner.execute()
```

##### Dag and run output

![dag](docs/workflow.png)
![log](docs/snakemake_workflow_output.png)

### Test

```bash
python -m pytest tests -s -o log_cli=True -vvvv
```


### TODO
- [ ] Refactor DAG.
- [ ] Web interface.
- [ ] Pbs/Torque executor
- [ ] More cache mode.
- [ ] Support run in Cloud platform.
- [ ] Run CWL script, Convert between CWL and flowsaber script.

### Reference
- [nextflow](https://github.com/nextflow-io/nextflow)
- [prefect](https://github.com/PrefectHQ/prefect)

==> flowsaber-0.1.3.3.tar.gz: flowsaber-0.1.3.3/README.md <==
# A dataflow based workflow framework.
### work in progress

<p align="center">

  

  
  <a href="https://pypi.python.org/pypi/flowsaber/">
    <img src="https://img.shields.io/pypi/v/flowsaber.svg" alt="Install with PyPi" />
  </a>
  
  
  <a href="https://github.com/zhqu1148980644/flowsaber/releases">
  	<img src="https://img.shields.io/github/v/release/zhqu1148980644/flowsaber?include_prereleases&label=github" alt="Github release">
  </a>
 
  <a href="https://zhqu1148980644.github.io/flowsaber/index.html">
  	<img src="https://readthedocs.org/projects/ansicolortags/badge/?version=latest" alt="Documentation">
  </a>
  
  <a href="https://pypi.python.org/pypi/flowsaber">
    <img src="https://img.shields.io/pypi/pyversions/flowsaber.svg" alt="Version">
  </a>
  
  <a href="https://pepy.tech/project/flowsaber">
    <img src="https://pepy.tech/badge/flowsaber" alt="Downloads">
  </a>

  <a href="https://pepy.tech/project/flowsaber">
    <img src="https://pepy.tech/badge/flowsaber/week" alt="Downloads per week">
  </a>
  
  <a href="https://github.com/zhqu1148980644/flowsaber/actions/workflows/python-package-conda.yml">
    <img src="https://github.com/zhqu1148980644/flowsaber/actions/workflows/python-package-conda.yml/badge.svg" alt="Build Status">
  </a>

  <a href="https://app.codecov.io/gh/zhqu1148980644/flowsaber">
    <img src="https://codecov.io/gh/zhqu1148980644/flowsaber/branch/main/graph/badge.svg" alt="codecov">
  </a>

  <a href="https://github.com/zhqu1148980644/flowsaber/blob/master/LICENSE">
    <img src="https://img.shields.io/github/license/zhqu1148980644/flowsaber" alt="license">
  </a>

</p>




### Features

- Dataflow-like task composing syntax inspired from [nextflow](https://github.com/nextflow-io/nextflow) 's DSL2.
- Python based: Import/Compose/Modify Task/Flow objects at any time.
- DAG generation.
- Local Cache.
- Clustering support based on ray.
- Conda and Container execution environment.


### Install

```bash
pip install flowsaber
```

### Example

- A minimum working example consists most features and usages of `flowsaber`.

```python
import matplotlib.image as mpimg
import matplotlib.pyplot as plt
import flowsaber
from flowsaber import *

@task
def add(self, num):  # self is optional
    return num + 1

@task
def multiply(num1, num2):
    return num1 * num2

@shell
def write(num):
    """echo {num} > 1.txt"""
    return '*.txt'

@task
def read(f: File):
    return open(str(f)).readlines()

@flow
def sub_flow(num):
    return add(num) | map_(lambda x: x ** 2) | add

@flow
def my_flow(num):
    [sub_flow(num), sub_flow(num)] | multiply \
    | write | read | flatten \
    | map_(lambda x: int(x.strip())) \
    | view

config.update({
    'cpu': 8,
    Task: {
        'executor': 'ray'
    }
})

# set input
num_ch = Channel.values(1, 2, 3, 4, 5, 6, 7, 8)
# resolve dependencies
workflow = my_flow(num=num_ch)
# now can generate dag
workflow.graph.render('quick_start_dag', view=False, format='png', cleanup=True)
# try run the flow
asyncio.run(flowsaber.run(workflow))

# visualize the flow
img = mpimg.imread('quick_start_dag.png')
imgplot = plt.imshow(img)
plt.show(block=False)
```

### Test

```bash
python -m pytest tests -s -o log_cli=True -vvvv
```


### TODO
- [ ] Refactor DAG.
- [ ] Web interface.
- [ ] Pbs/Torque executor
- [ ] More cache mode.
- [ ] Supportrun in Cloud platform.
- [ ] Run CWL script, Convert between CWL and flowsaber script.

### Reference
- [nextflow](https://github.com/nextflow-io/nextflow)
- [prefect](https://github.com/PrefectHQ/prefect)
